//+------------------------------------------------------------------+
//|                                          AdaptiveLaguerre_v3.mq4 |
//|                             Copyright  2008-13, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008-13, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  Tomato
#property indicator_width1  2  
#property indicator_color2  DeepSkyBlue
#property indicator_width2  2
#property indicator_color3  DeepSkyBlue
#property indicator_width3  2


extern int  TimeFrame            =     0;    //Time Frame in min
extern int  Price                =     0;    //Applied Price: 0-C,1-O,2-H,3-L,4-Median,5-Typical,6-Weighted
extern int  Length               =    10;    //Length  
extern int  Order                =     4;    //Laguerre Filter Order 
extern int  AdaptiveMode         =     1;    //Adaptive Mode:0-off,1-on 
extern int  AdaptiveSmooth       =     5;    //Adaptive Factor Smoothing Period
extern int  AdaptiveSmoothMode   =     4;    //Adaptive Factor Smoothing Mode:0-SMA,1-EMA,2-Wilder,3-LWMA,4-Median
extern int  ColorMode            =     1;    //Color Mode(0-off,1-on) 



double  laguerre[];
double  upTrend1[];
double  upTrend2[];
double  trend[];
double  price[];
double  diff[];
double  gamma[];


double   L[][2], ema[2];
datetime prevtime, ptime;
bool     ftime = true;
string   TF, IndicatorName, short_name;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void init()
{
   if(TimeFrame <= Period()) TimeFrame = Period();
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
    
//--- indicator buffers mapping 
   IndicatorBuffers(7);
   SetIndexBuffer(0,laguerre); SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(1,upTrend1); SetIndexStyle(1,DRAW_LINE);  
   SetIndexBuffer(2,upTrend2); SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(3,   trend);
   SetIndexBuffer(4,   price);
   SetIndexBuffer(5,    diff);
   SetIndexBuffer(6,   gamma);
//--- 
   TF = timeframeToString(TimeFrame);
   IndicatorName = WindowExpertName(); 
   short_name = IndicatorName+"["+TF+"]("+priceToString(Price)+","+Length+","+Order+","+AdaptiveMode+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,short_name+"("+Order+")"+" "+TF);
   SetIndexLabel(1,"UpTrend");
   SetIndexLabel(2,"DnTrend");
//---
   int draw_begin = Bars - iBars(NULL,TimeFrame)*TimeFrame/Period() + 2*Length + AdaptiveSmooth;
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);
   SetIndexDrawBegin(2,draw_begin);
   
   
   ArrayResize(L,Order);
 
//--- initialization done
}
int deinit() {Comment(""); return(0);}

//+------------------------------------------------------------------+
//| AdaptiveLaguerre_v3                                              |
//+------------------------------------------------------------------+
int start()
{
   int    limit, i, shift, counted_bars = IndicatorCounted(); 
   double sgamma;
   
   if(counted_bars > 0) limit=Bars-counted_bars-1;
   if(counted_bars < 0) return(0);
   if(counted_bars < 1)
   { 
   limit = Bars-1;   
      for(i=limit;i>=0;i--) 
      {
      laguerre[i] = EMPTY_VALUE; 
      upTrend1[i]  = EMPTY_VALUE; 
      upTrend2[i]  = EMPTY_VALUE; 
      }
   }   
   
   if(TimeFrame != Period())
	{
   limit = MathMax(limit,TimeFrame/Period());   
      
      for(shift = 0;shift < limit;shift++) 
      {	
      int y = iBarShift(NULL,TimeFrame,Time[shift]);
      
      
      double Laguerre   = iCustom(NULL,TimeFrame,IndicatorName,0,Price,Length,Order,AdaptiveMode,AdaptiveSmooth,AdaptiveSmoothMode,ColorMode,0,y);      
              
      laguerre[shift] = Laguerre;
         
         if(ColorMode > 0)
         {
         int rsitrend = iCustom(NULL,TimeFrame,IndicatorName,0,Price,Length,Order,AdaptiveMode,AdaptiveSmooth,AdaptiveSmoothMode,ColorMode,3,y);      
              
         upTrend1[shift] = EMPTY_VALUE;
         upTrend2[shift] = EMPTY_VALUE;
         if(rsitrend > 0) upTrend1[shift] = laguerre[shift]; 
         }
      } 
      
      if(ColorMode > 0)
      {
         for(shift = limit;shift >= 0;shift--) 
         {	
            if(laguerre[shift] > laguerre[shift+1] && TimeFrame > Period()) 
            {
            upTrend2[shift]   = laguerre[shift];
            upTrend2[shift+1] = laguerre[shift+1];
            }	
         }
      }   
	return(0);
	}
	else 
   {
      for(shift=limit;shift>=0;shift--)
      {
      price[shift] = iMA(NULL,0,1,0,0,Price,shift);
      
      if(shift > Bars - Length) continue;
      
      diff[shift] = MathAbs(price[shift] - laguerre[shift+1]);
      
      if(shift > Bars - 2*Length) continue;              
      
         if(AdaptiveMode > 0) 
         {
         gamma[shift] = AdaptiveGamma(diff,Length,shift);
      
            switch(AdaptiveSmoothMode)
            {
            case 1 : sgamma = EMA   (gamma[shift],AdaptiveSmooth    ,Time[shift],shift); break;
            case 2 : sgamma = EMA   (gamma[shift],2*AdaptiveSmooth-1,Time[shift],shift); break;  
            case 3 : sgamma = LWMA  (gamma       ,AdaptiveSmooth    ,shift); break;   
            case 4 : sgamma = Median(gamma       ,AdaptiveSmooth    ,shift); break; 
            default: sgamma = SMA   (gamma       ,AdaptiveSmooth    ,shift); break;
            }
         }
         else sgamma = 10.0/(Length+9);
    
      laguerre[shift] = Laguerre(price[shift],sgamma,Order,Time[shift],shift);
    
     
         if(ColorMode > 0)
         {
         trend[shift] = trend[shift+1];
         if(laguerre[shift] > laguerre[shift+1]) trend[shift] = 1;
         if(laguerre[shift] < laguerre[shift+1]) trend[shift] =-1;    
         
         upTrend1[shift] = EMPTY_VALUE;   
         upTrend2[shift] = EMPTY_VALUE;    
        
            if(trend[shift] > 0)
            {
               if(upTrend1[shift+1] == EMPTY_VALUE)
               {
                  if(upTrend1[shift+2] == EMPTY_VALUE) 
                  {
                  upTrend1[shift]   = laguerre[shift];
                  upTrend1[shift+1] = laguerre[shift+1];
                  upTrend2[shift]   = EMPTY_VALUE;
                  }
                  else 
                  {
                  upTrend2[shift]   = laguerre[shift];
                  upTrend2[shift+1] = laguerre[shift+1];
                  upTrend1[shift]   = EMPTY_VALUE;
                  }
               }
               else
               {
               upTrend1[shift]  = laguerre[shift];
               upTrend2[shift]  = EMPTY_VALUE;
               }
            }
         }
      }
   } 
//--- done       
   return(0);
}
//+------------------------------------------------------------------+
// Laguerre filter
double Laguerre(double price,double gamma,int order,datetime time,int bar)
{
   double gam = 1 - gamma; 
      
   double array[];
   ArrayResize(array,order);
   
   if(time != prevtime)
   {
   for(int i=0;i<order;i++) L[i][1] = L[i][0];
   prevtime = time;
   }
   
   for(i=0;i<order;i++)
   {
      if(bar >= Bars - order) L[i][0] = price;
      else
      {
         if(i == 0) L[i][0] = (1 - gam)*price + gam*L[i][1];
         else
         L[i][0] = -gam*L[i-1][0] + L[i-1][1] + gam*L[i][1];
        
      array[i] = L[i][0];
      }
   }
  
   return(TriMA_gen(array,order,0));
}


// SMA - Simple Moving Average
double SMA(double& array[],int per,int bar)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[bar+i];
     
   return(Sum/per);
}                
// EMA - Exponential Moving Average
double EMA(double price,int per,datetime time,int bar)
{
   if(ptime != time) {ema[1] = ema[0]; ptime = time;} 
   
   if(ftime) {ema[0] = price; ftime = false;}
   else 
   ema[0] = ema[1] + 2.0/(1+per)*(price - ema[1]); 
   
   return(ema[0]);
}

// LWMA - Linear Weighted Moving Average 
double LWMA(double& array[],int per,int bar)
{
   double lwma, Sum = 0, Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= (per - i);
      Sum += array[bar+i]*(per - i);
      }
   
   if(Weight>0) lwma = Sum/Weight; else lwma = 0; 
   
   return(lwma);
} 
// TriMA generalized
double TriMA_gen(double& array[],int per,int bar)
{
   int len1 = MathFloor((per+1)*0.5);
   int len2 = MathCeil ((per+1)*0.5);
   double sum=0;
   for(int i = 0;i < len2;i++) sum += SMA(array,len1,bar+i);
   double trimagen = sum/len2;
   
   return(trimagen);
}

// Median - Moving Median
double Median(double& price[],int per,int bar)
{
   double median, array[];
   ArrayResize(array,per);
   
   for(int i = 0; i < per;i++) array[i] = price[bar+i];
   ArraySort(array);
   
   int num = MathRound((per-1)/2); 
   if(MathMod(per,2) > 0) median = array[num]; else median = 0.5*(array[num]+array[num+1]);
   
   return(median); 
}

// Adaptive Factor
double AdaptiveGamma(double& array[],int per,int bar)
{
   double eff, sum = 0, max = 0, min = 1000000000; 
   for(int i = 0;i < per;i++) 
   {
   if(array[bar+i] > max) max = array[bar+i]; 
   if(array[bar+i] < min) min = array[bar+i]; 
   }
      
   if(max-min != 0) eff = (array[bar] - min)/(max - min); else eff = 0;
   
   return(eff);  
} 



string timeframeToString(int TF)
{
   switch(TF)
   {
   case PERIOD_M1       : return("M1");   
   case PERIOD_M5       : return("M5");      
   case PERIOD_M15      : return("M15");
   case PERIOD_M30      : return("M30");
   case PERIOD_H1       : return("H1");
   case PERIOD_H4       : return("H4");
   case PERIOD_D1       : return("D1");
   case PERIOD_W1       : return("W1");
   case PERIOD_MN1      : return("MN1");      
   default              : return("Current");
   }
}

string priceToString(int app_price)
{
   switch(app_price)
   {
   case PRICE_CLOSE   :    return("Close");
   case PRICE_HIGH    :    return("High");
   case PRICE_LOW     :    return("Low");
   case PRICE_MEDIAN  :    return("Median");
   case PRICE_OPEN    :    return("Open");
   case PRICE_TYPICAL :    return("Typical");
   case PRICE_WEIGHTED:    return("Weighted");
   default            :    return("");
   }
}


